#       _                         __    __           __   __      __         __
#      (_)___  ____  ____  __  __/ /_  / /___ ______/ /__/ /___ _/ /_  ___  / /
#     / / __ \/ __ \/ __ \/ / / / __ \/ / __ `/ ___/ //_/ / __ `/ __ \/ _ \/ / 
#    / / /_/ / / / / / / / /_/ / /_/ / / /_/ / /__/ ,< / / /_/ / /_/ /  __/ /  
# __/ /\____/_/ /_/_/ /_/\__, /_.___/_/\__,_/\___/_/|_/_/\__,_/_.___/\___/_/   
#/___/                  /____/                                                 
#
#
# author: Johannes Kunze
# web: http://www.jonnyblacklabel.de/
# twitter: @jonnyblacklabel
#

from .endpoint_base import BaseEndpoint
from .method_base import BaseMethod, MethodParams, AcceptsKeywordFilter, AcceptsDateFilter

class Keyword(BaseEndpoint):
	"""Keyword Endpoint
	
	metrics.tools keyword endpoint.

	Call keyword('example') to set keyword.

	Methods:
		- details
		- related
	
	Attributes
	----------
	endpoint : {str}
		Last part of API url
	"""
	endpoint = 'keyword'
	
	def __init__(self, client, *args, **kwargs):
		self.details = Details(client, self.endpoint)
		self.related = Related(client, self.endpoint)

	def __call__(self, keyword):
		if len(keyword) > 0:
			self.details.params.keyword = keyword
			self.related.params.keyword = keyword
		else:
			raise ValueError('Input can not be empty.')
		return self


class Details(BaseMethod):
    """Details Method
    
    metrics.tools method for keyword.details.
    Returns searchvolume, cpc and competition for given keyword.
    
    Attributes
    ----------
    params : {MethodParams}
        Definition for fixed and needed parameters
    """

    params = MethodParams(fixed={'query':'details'},needed=['keyword'])

    def __init__(self, client, endpoint):
        super().__init__(client, endpoint)


class Related(BaseMethod):
    """Related Method
    
    metrics.tools method for keyword.related.
    Returns related keyword for given keyword.
    
    Attributes
    ----------
    params : {MethodParams}
        Definition for fixed and needed parameters
    """

    params = MethodParams(fixed={'query':'related'},needed=['keyword'],optional=['details'])

    def __init__(self, client, endpoint):
        super().__init__(client, endpoint)

    def details(self, degree):
        """Set detail degree
        
        Set the detail degree for related keywords.
        Represents the "schnell-umfassend" controle in the frontend.
        
        Parameters
        ----------
        degree : {string}
            Filter by degree, default: 3, min 0 max 3
        
        Returns
        -------
        self
        """
        self.params.details = degree
        return self

if __name__ == '__main__':
	pass


from .endpoint_base import BaseEndpoint
from .method_base import BaseMethod, MethodParams, AcceptsOrderColumn, AcceptsDateFilter, Paginatable

class Url(BaseEndpoint):
    """Url Endpoint
    
    metrics.tools url endpoint.

    Call url('https://www.example.com/') to set url.

    Methods:
        - rankings
    
    Attributes
    ----------
    endpoint : {str}
        Last part of API url
    """
    endpoint = 'url'
    
    def __init__(self, client, *args, **kwargs):
        self.rankings = Rankings(client, self.endpoint)

    def __call__(self, url):
        if len(url) > 0:
            self.rankings.params.url = url
        else:
            raise ValueError('Input can not be empty.')
        return self


class Rankings(AcceptsOrderColumn, AcceptsDateFilter, Paginatable, BaseEndpoint):
    """Rankings method
    
    metrics.tools method for url.rankings.
    Returns rankings for given url.
    
    Attributes
    ----------
    params : {MethodParams}
        Definition for fixed and needed parameters
    """

    params = MethodParams(fixed={'query':'rankings'},needed=['url'],optional=['limit','offset','order_column','order_direction'])

    def __init__(self, client, endpoint):
        super().__init__(client, endpoint)


if __name__ == '__main__':
    pass



from .errors import ApiError

class ClientResponse():
	"""Response object
	
	Returned response object.

	Json response is parsed into response object.

	example_domain.values → if values are present
	example_domain.json → parsed json response
	example_domain.result → 'success' or 'error'
	"""

	def __init__(self, response):
		self._response = response

		if len(self._response.text) > 0:
			self.json = self._response.json()

			for attr, value in self.json.items():
				if value:
					setattr(self, attr, value)
		else:
			self._response.status_code = 204
			self.result = 'error'
			self.error_msg = 'No data'

	def __len__(self):
		return len(getattr(self, 'values', ''))

	def raise_for_status(self):
		"""Raises Exception
		
		Raises
		------
		ApiError
		"""
		if self.result == 'error':
			raise ApiError(f"Status Code: {self._response.status_code}, Error: {getattr(self, 'result', 'No error')}, Message: {getattr(self, 'error_msg', 'No message')}")
		else:
			pass

	def __repr__(self):
		return f'<metrics.tools Response [{self._response.status_code}]>'


if __name__ == '__main__':
	pass


from .endpoint_base import BaseEndpoint
from .method_base import BaseMethod, MethodParams, AcceptsOrderColumn, AcceptsDateFilter, AcceptsKeywordFilter, Paginatable

class Domain(BaseEndpoint):
	"""Domain Endpoint
	
	metrics.tools domain endpoint.

	Call domain('example.com') to set domain.

	Methods:
		- sk [visibility]
		- rankings
		- position
	
	Attributes
	----------
	endpoint : {str}
		Last part of API url
	"""
	endpoint = 'domain'
	
	def __init__(self, client, *args, **kwargs):
		self.sk = Sk(client, self.endpoint)
		self.rankings = Rankings(client, self.endpoint)
		self.position = Position(client, self.endpoint)

	def __call__(self, domain):
		if len(domain) > 0:
			self.sk.params.domain = domain
			self.rankings.params.domain = domain
			self.position.params.domain = domain
		else:
			raise ValueError('Input can not be empty.')
		return self


class Sk(AcceptsDateFilter, BaseMethod):
    """Visibility Method
    
    metrics.tools method for domain.sk.
    Returns the visibility for given domain.
    
    Attributes
    ----------
    params : {MethodParams}
        Definition for fixed, needed and optional parameters
    """

    params = MethodParams(fixed={'query':'sk'},needed=['domain'],optional=['type','date'])
     
    def __init__(self, client, endpoint):
        super().__init__(client, endpoint)

    def current(self):
        """Current visibility
        
        Filters API call for latest visibility score
        
        Returns
        -------
        self
        """
        self.params.type = 'current'
        return self

    def all(self):
        """Get all
        
        Run API call for all visibility values
        
        Returns
        -------
        ClientResponse
            metrics.tools response object
        """
        self.params.type = 'all'
        return self.get()

    def minmax(self):
        """Set minmax
        
        Filter API call for min and max historical visibility value
        
        Returns
        -------
        self
        """
        self.params.type = 'minmax'
        return self


class Rankings(AcceptsOrderColumn, AcceptsDateFilter, Paginatable):
    """Rankings Method
    
    metrics.tools method for domain.rankings.
    Returns rankings for given domain.
    
    Attributes
    ----------
    params : {MethodParams}
        Definition for fixed, needed and optional parameters
    """

    params = MethodParams(fixed={'query':'rankings'},needed=['domain'],optional=['limit','offset','order_column','order_direction'])

    def __init__(self, client, endpoint):
        super().__init__(client, endpoint)


class Position(AcceptsKeywordFilter, AcceptsDateFilter, BaseMethod):
    """Position Method
    
    metrics.tools method for domain.position.
    Returns ranking position for given domain for given keyword.
    
    Attributes
    ----------
    params : {MethodParams}
        Definition for fixed, needed and optional parameters
    """

    params = MethodParams(fixed={'query':'position'},needed=['domain','keyword'],optional=['date'])

    def __init__(self, client, endpoint):
        super().__init__(client, endpoint)

if __name__ == '__main__':
	pass


#       _                         __    __           __   __      __         __
#      (_)___  ____  ____  __  __/ /_  / /___ ______/ /__/ /___ _/ /_  ___  / /
#     / / __ \/ __ \/ __ \/ / / / __ \/ / __ `/ ___/ //_/ / __ `/ __ \/ _ \/ / 
#    / / /_/ / / / / / / / /_/ / /_/ / / /_/ / /__/ ,< / / /_/ / /_/ /  __/ /  
# __/ /\____/_/ /_/_/ /_/\__, /_.___/_/\__,_/\___/_/|_/_/\__,_/_.___/\___/_/   
#/___/                  /____/                                                 
#
#
# author: Johannes Kunze
# web: http://www.jonnyblacklabel.de/
# twitter: @jonnyblacklabel
#

from .domain import Domain
from .keyword import Keyword
from .url import Url
from .rest_client import RestClient

class MetricsTools():
	"""metrics.tools API
	
	Run calls for metrics.tools API

	Usage
	=====
	from metrics_tools import MetricsTools
	
	api_key = 'your api key'
	mt = MetricsTools(api_key, verbose=True)

	example_domain = mt.domain('example.com')

	visibility = example_domain.sk.get()
	rankings = example_domain.rankings.get()

	keyword_details = mt.keyword('api').details.get()

	Response examples
	=================
	example_domain.values → if values are present
	example_domain.json → parsed json response
	example_domain.result → 'success' or 'error'

	API calls
	=========
	Final API call is made through .get(), .all() or .take() if possible.
	Returns response object with attributes.
	The minimum interval for calls is 0.2 sec.
	"""

	def __init__(self, api_key, *args, **kwargs):
		self.__client = RestClient(api_key, *args, **kwargs)
		self.domain = Domain(self.__client, *args, **kwargs)
		self.keyword = Keyword(self.__client, *args, **kwargs)
		self.url = Url(self.__client, *args, **kwargs)

		

if __name__ == '__main__':
	pass


import jsonpickle
import datetime


class BaseParameter():

    def __init__(self, name):
        self.name = name

    def __get__(self, obj, owner):
        return getattr(obj, self.name, None)

    def __set__(self, obj, value):
        setattr(obj, self.name, value)


def acceptsValues(values):
    def decorator(func):
        def wrapper(*args, **kwwargs):
            if isinstance(values, (list, )):
                if args[1] in values:
                    return func(*args, **kwargs)
                else:
                    raise ValueError(f'Must be one of {values}')
            else:
                raise TypeError(f'Valid values musst be of type {type(list)}')
        return wrapper
    return decorator


def acceptsWeekday(weekday):
    def decorator(func):
        def wrapper(*args, **kwargs):
            if isinstance(args[1], datetime.date):
                date = args[1]
                args[1] = datetime.datetime.strftime(date, '%Y-%m-%d')
            else:
                date = datetime.datetime.strptime(args[1], '%Y-%m-%d').date()
            if date.weekday() != weekday:
                raise ValueError(f'Musst be day {weekday}, {date.weekday()} was given')
            return func(*args, **kwargs)
        return wrapper
    return decorator


class MethodParams():
    
    def __init__(self, fixed, needed, optional=None):
        self._fixed = fixed
        self._needed = needed
        self._optional = optional  
        self.reset()

    @property
    def fixed(self):
        return self._fixed

    @property
    def needed(self):
        return self._needed

    @property
    def optional(self):
        return self._optional

    def get(self):
        """Get parameters
        
        Builds dict from parameters
        
        Returns
        -------
        dict
            Parameters for API call
        """
        return jsonpickle.decode(jsonpickle.encode(self, unpicklable=False))

    def check_needed(self):
        """Check needed
        
        Checks if all needed parameters are present.
        
        Raises
        ------
        ValueError
            Error if needed parameter is not given.
        """
        for need in self.needed:
            if getattr(self, need, None) == None:
                raise ValueError(f'{need} can not be {type(None)}')


    def reset(self):
        """Reset parameters

        Reset the optional parameters
        """
        if self.optional is not None:
            for value in self.optional:
                setattr(self, value, None)
        for value in self.needed:
            setattr(self, value, getattr(self, value, None))
        for item,value in self.fixed.items():
            setattr(self, item, value)
        
    def __getstate__(self):
        state = self.__dict__.copy()
        for attr in ['_fixed','_needed','_optional']:
            del state[attr]
        return state


class BaseMethod():

    def __init__(self, client, endpoint, *args, **kwargs):
        self._endpoint = endpoint
        self._client = client

    def __repr__(self):
        return f'<metrics.tools Method [{self.params.query}]>'

    def get(self):
        """Get result
        
        Run API call for method
        
        Returns
        -------
        ClientResponse
            metrics.tools response object
        """
        self.params.check_needed()
        self._client.params = self.params.get()
        self.params.reset()
        return self._client.get(self._endpoint)


class Paginatable(BaseMethod):

    def get(self, offset=0, limit=10):
        """Get result by offset and limit
        
        Run API call for method
        
        Parameters
        ----------
        offset : {number}, optional
            Paginationoffset (the default is 0)
        limit : {number}, optional
            Resultlimit, default 10, maximum 1000 (the default is 10)
        
        Returns
        -------
        ClientResponse
            metrics.tools response object
        """
        self.params.check_needed()
        self._client.params = {**self.params.get(), **{'offset':offset,'limit':limit}}
        self.params.reset()
        return self._client.get(self._endpoint)

    def all(self, **kwargs):
        """Get all results
        
        Run paginated API call for method
        
        Returns
        -------
        ClientResponse
            metrics.tools response object
        """
        num = kwargs.get('num', float('inf'))
        limit = num if num < 1000 else 1000
        offset = 0
        iterations = 1
        self.params.check_needed()
        while True:
            if iterations * limit <= offset or num <= offset:
                self.params.reset()
                break
            self._client.params = {**self.params.get(), **{'offset':offset,'limit':limit}}
            response = self._client.get(self._endpoint)
            if response.result == 'success' and len(response) > 0: #result present, no error
                if offset == 0: #first run
                    if hasattr(response, 'n_all'): #calc interations for all results
                        x = divmod(response.n_all, limit)
                        iterations = x[0] + 1 if x[1] > 0 else x[0]
                    result = response
                else:
                    result.__dict__['values'].extend(response.__dict__['values'])
                if iterations == 1: break
            else:
                result = response
                self.params.reset()
                break
            offset += limit
        return result

    def take(self, num):
        """Get first n results by num
        
        Run paginated API call for method for first n results
        
        Returns
        -------
        ClientResponse
            metrics.tools response object
        """
        return self.all(num=num)


class AcceptsOrderColumn():

    def orderby(self, column, direction='desc'):
        """Sort result
        
        Sort result by column and direction
        
        Parameters
        ----------
        column : {string}
            Sort by column, default: traffic, possible values: position, searchvolume, traffic
        direction : {str}, optional
            Sortierungsrichtung, default: desc, mögliche Werte: asc, desc (the default is 'desc')
        
        Returns
        -------
        self
        """
        self.params.order_column = column
        self.params.order_direction = direction
        return self


class AcceptsDateFilter():

    @acceptsWeekday(0)
    def from_(self, date):
        """Set date
        
        Filters API call for specific date
        
        Parameters
        ----------
        date : {string or datetime.date}
            - needs to be monday
            - string format 'yyyy-mm-dd'
        
        Returns
        -------
        self
        """
        self.params.date = date
        return self

class AcceptsKeywordFilter():

    def for_keyword(self, keyword):
        """Set Keyword
        
        Filter API call for given keyword.

        Parameters
        ----------
        keyword : {string}
            Keyword to filter for
        
        Returns
        -------
        self
        """
        self.params.keyword = keyword
        return self

if __name__ == '__main__':
    pass


class TgPyErrorBase(Exception):
	pass


class ApiError(TgPyErrorBase):
	def __init__(self, answer, msg=None):
		if msg is None:
			msg = f'Api answered with an error: {answer}'
		super(ApiError, self).__init__(msg)


if __name__ == '__main__':
	pass


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="metrics_tools",
    version="0.0.1",
    author="Jonnyblacklabel",
    author_email="mail@jonnyblacklabel.de",
    description="Package for making calls to metrics.tools API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jonnyblacklabel/metrics_tools",
    packages=setuptools.find_packages(),
    install_requires=['requests','jsonpickle'],
    python_requires='>=3.6',
    classifiers=(
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)


import requests
from .response import ClientResponse 

def verbosable(func):
	def wrapper(*args, **kwargs):
		if args[0].verbose:
			print('/'.join([args[0].api_url, args[1]]))
			print(args[0].params)
		return func(*args, **kwargs)
	return wrapper


class RestClient():

	api_url = 'https://metrics.tools/metricstools/api'

	def __init__(self, api_key, *args, **kwargs):
		self.verbose = kwargs.get('verbose', None)
		self.session = requests.Session()
		self.__api_key = api_key
		self.params = {}

	@property
	def params(self):
		return {**self.__params, **{'apikey':self.__api_key}}

	@params.setter
	def params(self, params):
		if isinstance(params, dict):
			params = dict((k,v) for k,v in params.items() if v is not None)
			self.__params = params
		else:
			raise TypeError('Params need to be of type dict!')


	@verbosable
	def get(self, endpoint):
		response = self.session.get('/'.join([self.api_url, endpoint]), params=self.params)
		return ClientResponse(response)


if __name__ == '__main__':
	pass